<?php
//_________________________________________________________________
// 
//          Project: wepco.de
//           Source: Folder_Matic.class.php
//_________________________________________________________________
//
//        CreatedBy: Enrico Hoeffler
//      DateCreated: 2007-09-20 16:17:04 +0200 (Do, 20 Sep 2007)
//
//          Version: 2.0.0
//        $Revision: 209 $
//   $LastChangedBy: Enrico $
// $LastChangedDate: 2008-10-20 18:40:55 +0200 (Mo, 20 Okt 2008) $
//      PHP-Version: 5.2.5
//
//      Description: This class reads the folder- and file-structure
//                   of a directory.
//_________________________________________________________________
//
//   copyright  2004-2008 by wepco.de - all rights reserved
//_________________________________________________________________

class Folder_Matic {

	var $log_enable = FALSE;

	function Picture_Matic() {
		Logger :: staticDebug($this->log_enable, "Initialize Folder Matic ...");
	}

	function getFolder($dir, $folder_name) {

		$sub_folder = $this->getRequestSubFolder();
		Logger :: staticDebug($this->log_enable, "Subfolder: " . $sub_folder);

		$folder = $this->getFolderDetails($dir, $folder_name, $sub_folder);

		Logger :: staticDebug($this->log_enable, "... Searching Sub Folders");
		$sub_folders = $this->getSubFolders($dir, $folder_name, $sub_folder);
		$folder['sub_folders'] = $sub_folders;
		Logger :: staticDebug($this->log_enable, "Found " . sizeof($sub_folders) . " Sub Folder(s)");

		Logger :: staticDebug($this->log_enable, "... Searching Files");
		$files = $this->getFiles($folder['full_path']);
		$folder['files'] = $files;
		Logger :: staticDebug($this->log_enable, "Found " . sizeof($files) . " File(s)");

		return $folder;
	}

	/**
	 * Checks request for subfolder information.
	 * 
	 * @return subfolder
	 */
	private function getRequestSubFolder() {
		foreach ($_REQUEST AS $key => $value) {
			return $key;
		}
		return "";
	}

	/**
	 * Reads folder-details of the folder directory and returns an array of deatails.
	 * 
	 * @return array of Album
	 */
	private function getFolderDetails($dir, $main_folder, $sub_folder) {
		$folder_name = $main_folder;
		$full_path = "";
		$sub_path = "";
		$folder_info_desc = "";
		$folder_info_name = "";

		//prepare path
		if (!empty ($sub_folder)) {
			$sub_path .= $sub_folder . DIRECTORY_SEPARATOR;
			$folder_name = $sub_folder;
		}

		$folder_name = $this->extractFolderName($folder_name);

		$full_path = $dir . $main_folder . DIRECTORY_SEPARATOR . $sub_path;

		$sub_path = $this->correctPath($sub_path);
		$full_path = $this->correctPath($full_path);

		if (file_exists($full_path . "info.txt")) {
			include ($full_path . "info.txt");
		}

		if (empty ($folder_info_name) || $folder_info_name == "") {
			$folder_info_name = $folder_name;
		}

		$retVal['name'] = $folder_name;
		Logger :: staticDebug($this->log_enable, "[folder] name: " . $folder_name);
		$retVal['full_path'] = $full_path;
		Logger :: staticDebug($this->log_enable, "[folder] full_path: " . $full_path);
		$retVal['sub_path'] = $sub_path;
		Logger :: staticDebug($this->log_enable, "[folder] sub_path: " . $sub_path);
		$retVal['info_name'] = $folder_info_name;
		Logger :: staticDebug($this->log_enable, "[folder] folder: " . $folder_info_name);
		$retVal['info_desc'] = $folder_info_desc;
		Logger :: staticDebug($this->log_enable, "[folder] info: " . $folder_info_desc);

		return $retVal;
	}

	/**
	 * Reads sub-folders of the folder-directory and returns an array of Folders.
	 * 
	 * @return array of Album
	 */
	private function getSubFolders($dir, $folder_name, $sub_folder) {
		$full_path = "";

		//prepare path
		$full_path = $dir . DIRECTORY_SEPARATOR . $folder_name . DIRECTORY_SEPARATOR;
		if (!empty ($sub_folder)) {
			$full_path .= $sub_folder . DIRECTORY_SEPARATOR;
		}

		$full_path = $this->correctPath($full_path);

		Logger :: staticDebug($this->log_enable, "read albums from directory " . $full_path);

		if (is_dir($full_path) && $filehandle = opendir($full_path)) {

			while (false !== ($content = readdir($filehandle))) {
				Logger :: staticDebug($this->log_enable, "check if is directory " . $content);
				if (is_dir($full_path . DIRECTORY_SEPARATOR . $content) && $content != "." && $content != ".." && substr($content, 0, 1) != "_") {
					$folders[] = $content;
				}
			}

			closedir($filehandle);
			if ($folders == null) {
				return;
			}
			natcasesort($folders);

			foreach ($folders as $folder) {
				Logger :: staticDebug($this->log_enable, "read folder from directory: " . $folder);
				$new_sub_folder = $sub_folder . DIRECTORY_SEPARATOR . $folder;
				$new_sub_folder = $this->correctPath($new_sub_folder);
				$retValue[] = $this->getFolderDetails($dir, $folder_name, $new_sub_folder);
			}
		}

		return $retValue;
	}

	/**
	 * Reads file-structure of the clicked picture-folder and returns an array of Images in an Album.
	 * 
	 * @return album
	 */
	private function getFiles($folder_dir) {

		$folder_dir = $this->correctPath($folder_dir);

		if (is_dir($folder_dir)) {
			Logger :: staticDebug($this->log_enable, "File path: " . $folder_dir);

			$dir = opendir($folder_dir);

			// Reading and checking images in the folder of the album
			while ($file = readdir($dir)) {
				if ($file != "." && $file != "..") {
					Logger :: staticDebug($this->log_enable, "[album] isFile: " . is_file($folder_dir . $file));
					if (is_file($folder_dir . $file) && $file != 'info.txt') {
						$files[$file] = $this->getFileDetails($folder_dir, $file);
					}
				}
			}

			if (!empty ($files)) {
				ksort($files);
			}
			$files;
		}
		return $files;
	}

	private function getFileDetails($dir, $file_name) {

		$filesize = round((filesize($dir . $file_name) / 1024), 1) . "kB";

		$file_info_name = "";
		$file_info_desc = "";
		if (file_exists($dir . "info.txt")) {
			include ($dir . "info.txt");
			$info_name = str_replace('.', '_', $file_name) . '_name';
			$file_info_name = $$info_name;

			$info_desc = str_replace('.', '_', $file_name) . '_desc';
			$file_info_desc = $$info_desc;

		}

		if (empty ($file_info_name) || $file_info_name == "") {
			$file_info_name = $file_name;
		}

		$file_type = $this->getFileType($file_name);

		$retVal['name'] = $file_name;
		Logger :: staticDebug($this->log_enable, "[file] name: " . $file_name);
		$retVal['type'] = $file_type;
		Logger :: staticDebug($this->log_enable, "[file] type: " . $file_type);
		$retVal['size'] = $filesize;
		Logger :: staticDebug($this->log_enable, "[file] size: " . $filesize);
		$retVal['info_name'] = $file_info_name;
		Logger :: staticDebug($this->log_enable, "[file] info_name: " . $file_info_name);
		$retVal['info_desc'] = $file_info_desc;
		Logger :: staticDebug($this->log_enable, "[file] info_desc: " . $file_info_desc);

		return $retVal;
	}

	private function getFileType($file_name) {
		//		$extension = substr($file, -4);
		//		if (($extension == ".JPG") || ($extension == ".jpg")) {}

		preg_match("|\.([a-z0-9]{2,4})$|i", $file_name, $fileSuffix);

		switch (strtolower($fileSuffix[1])) {
			case "js" :
				return "application/x-javascript";

			case "json" :
				return "application/json";

			case "jpg" :
			case "jpeg" :
			case "jpe" :
				//				return "image/jpg";
				return "img";

			case "png" :
			case "gif" :
			case "bmp" :
			case "tiff" :
				//				return "image/" . strtolower($fileSuffix[1]);
				return "img";

			case "css" :
				return "text/css";

			case "xml" :
				return "application/xml";

			case "doc" :
			case "docx" :
				return "application/msword";

			case "xls" :
			case "xlt" :
			case "xlm" :
			case "xld" :
			case "xla" :
			case "xlc" :
			case "xlw" :
			case "xll" :
				return "application/vnd.ms-excel";

			case "ppt" :
			case "pps" :
				return "application/vnd.ms-powerpoint";

			case "rtf" :
				return "application/rtf";

			case "pdf" :
				//				return "application/pdf";
				return "pdf";

			case "html" :
			case "htm" :
			case "php" :
				return "text/html";

			case "txt" :
				//				return "text/plain";
				return "text";

			case "mpeg" :
			case "mpg" :
			case "mpe" :
				return "video/mpeg";

			case "mp3" :
				return "audio/mpeg3";

			case "wav" :
				return "audio/wav";

			case "aiff" :
			case "aif" :
				return "audio/aiff";

			case "avi" :
				return "video/msvideo";

			case "wmv" :
				return "video/x-ms-wmv";

			case "mov" :
				return "video/quicktime";

			case "zip" :
				return "zip";
				//				return "application/zip";

			case "tar" :
				return "application/x-tar";

			case "swf" :
				return "application/x-shockwave-flash";

			default :
				if (function_exists("mime_content_type")) {
					$fileSuffix = mime_content_type($file_name);
				}

				return "unknown/" . trim($fileSuffix[0], ".");
		}
	}

	/**
	 * Reads file-structure of the clicked picture-folder and returns an array of Images in an Album.
	 * 
	 * @param $img current picture
	 * @param $albumpath relative path to the picture-folder
	 * @param $imagenumber the imagenumber to read details
	 * 
	 * @return album
	 */
	private function getImageDetails($img, $albumpath, $imagenumber) {
		$image['name'] = $img;
		Logger :: staticDebug($this->log_enable, "[image] name: " . $img);

		$imgsize = getimagesize(rtrim($albumpath . $img));
		$image['height'] = ($imgsize[1]);
		Logger :: staticDebug($this->log_enable, "[image] height: " . ($imgsize[1]));
		$image['width'] = ($imgsize[0]);
		Logger :: staticDebug($this->log_enable, "[image] width: " . ($imgsize[0]));
		$imagesize = $imgsize[0] . "x" . $imgsize[1];
		$image['resolution'] = $imagesize . "px";
		Logger :: staticDebug($this->log_enable, "[image] resolution: " . $imagesize . "px");

		$image['number'] = $imagenumber;
		Logger :: staticDebug($this->log_enable, "[image] number: " . $imagenumber);

		return $image;
	}

	private function correctPath($path) {
		$path = str_replace('\\', '/', $path);
		$path = str_replace('//', '/', $path);

		if ($path[0] == "/") {
			$path = substr($path, 1);
		}

		return $path;
	}

	private function extractFolderName($folder_name) {

		$length = strlen($folder_name);

		Logger :: staticDebug($this->log_enable, "extractFolderName - name: " . $folder_name);
		if ($folder_name[$length -1] == "/") {
			$folder_name = substr($folder_name, 0, $length -1);
			$length = strlen($folder_name);
			Logger :: staticDebug($this->log_enable, "cut /: " . $folder_name);
		}

		$start = 0;
		$end = $length;
		for ($i = 0; $length > $i; $i++) {
			if ($folder_name[$length - $i] == "/") {
				$start = $length - $i +1;
				break;
			}
		}

		$name = substr($folder_name, $start, $end);

		return $name;

	}
}
?>